@@ -1,4 +1,4 @@
-/* $OpenBSD: netcat.c,v 1.228 2024/08/05 07:16:30 tb Exp $ */
+/* $OpenBSD: netcat.c,v 1.229 2024/11/02 17:19:27 tb Exp $ */
 /*
  * Copyright (c) 2001 Eric Jackson <ericj@monkey.org>
  * Copyright (c) 2015 Bob Beck.  All rights reserved.
@@ -1387,7 +1387,7 @@ fdpass(int nfd)
 	memset(&cmsgbuf, 0, sizeof(cmsgbuf));
 	memset(&iov, 0, sizeof(iov));
 
-	mh.msg_control = (caddr_t)&cmsgbuf.buf;
+	mh.msg_control = &cmsgbuf.buf;
 	mh.msg_controllen = sizeof(cmsgbuf.buf);
 	cmsg = CMSG_FIRSTHDR(&mh);
 	cmsg->cmsg_len = CMSG_LEN(sizeof(int));
