@@ -1,4 +1,4 @@
-/*	$OpenBSD: crypto_arch.h,v 1.1 2024/08/11 13:02:39 jsing Exp $ */
+/*	$OpenBSD: crypto_arch.h,v 1.2 2025/02/14 12:01:58 jsing Exp $ */
 /*
  * Copyright (c) 2024 Joel Sing <jsing@openbsd.org>
  *
@@ -23,6 +23,15 @@
 #define HAVE_AES_ENCRYPT_INTERNAL
 #define HAVE_AES_DECRYPT_INTERNAL
 
+#define HAVE_SHA1_BLOCK_DATA_ORDER
+#define HAVE_SHA1_BLOCK_GENERIC
+
+#define HAVE_SHA256_BLOCK_DATA_ORDER
+#define HAVE_SHA256_BLOCK_GENERIC
+
+#define HAVE_SHA512_BLOCK_DATA_ORDER
+#define HAVE_SHA512_BLOCK_GENERIC
+
 #endif
 
 #endif
