@@ -1,4 +1,4 @@
-/* $OpenBSD: bn_exp.c,v 1.38 2023/03/15 04:30:20 jsing Exp $ */
+/* $OpenBSD: bn_exp.c,v 1.58 2025/02/13 11:15:09 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -120,67 +120,72 @@
 /* maximum precomputation table size for *variable* sliding windows */
 #define TABLE_SIZE	32
 
-/* this one works - simple but works */
+/* Calculates r = a^p by successive squaring of a. Not constant time. */
 int
 BN_exp(BIGNUM *r, const BIGNUM *a, const BIGNUM *p, BN_CTX *ctx)
 {
-	int i, bits, ret = 0;
-	BIGNUM *v, *rr;
+	BIGNUM *rr, *v;
+	int i;
+	int ret = 0;
 
 	if (BN_get_flags(p, BN_FLG_CONSTTIME) != 0) {
-		/* BN_FLG_CONSTTIME only supported by BN_mod_exp_mont() */
 		BNerror(ERR_R_SHOULD_NOT_HAVE_BEEN_CALLED);
 		return -1;
 	}
 
 	BN_CTX_start(ctx);
-	if ((r == a) || (r == p))
+
+	if ((v = BN_CTX_get(ctx)) == NULL)
+		goto err;
+
+	rr = r;
+	if (r == a || r == p)
 		rr = BN_CTX_get(ctx);
-	else
-		rr = r;
-	v = BN_CTX_get(ctx);
-	if (rr == NULL || v == NULL)
+	if (rr == NULL)
 		goto err;
 
-	if (BN_copy(v, a) == NULL)
+	if (!BN_one(rr))
 		goto err;
-	bits = BN_num_bits(p);
-
 	if (BN_is_odd(p)) {
-		if (BN_copy(rr, a) == NULL)
-			goto err;
-	} else {
-		if (!BN_one(rr))
+		if (!bn_copy(rr, a))
 			goto err;
 	}
 
-	for (i = 1; i < bits; i++) {
+	if (!bn_copy(v, a))
+		goto err;
+
+	for (i = 1; i < BN_num_bits(p); i++) {
 		if (!BN_sqr(v, v, ctx))
 			goto err;
-		if (BN_is_bit_set(p, i)) {
-			if (!BN_mul(rr, rr, v, ctx))
-				goto err;
-		}
+		if (!BN_is_bit_set(p, i))
+			continue;
+		if (!BN_mul(rr, rr, v, ctx))
+			goto err;
 	}
+
+	if (!bn_copy(r, rr))
+		goto err;
+
 	ret = 1;
 
-err:
-	if (r != rr && rr != NULL)
-		BN_copy(r, rr);
+ err:
 	BN_CTX_end(ctx);
-	return (ret);
+
+	return ret;
 }
+LCRYPTO_ALIAS(BN_exp);
 
 /* The old fallback, simple version :-) */
 int
 BN_mod_exp_simple(BIGNUM *r, const BIGNUM *a, const BIGNUM *p, const BIGNUM *m,
     BN_CTX *ctx)
 {
-	int i, j, bits, ret = 0, wstart, wend, window, wvalue;
+	int i, j, bits, wstart, wend, window, wvalue;
 	int start = 1;
-	BIGNUM *d;
+	BIGNUM *d, *q;
 	/* Table of variables obtained from 'ctx' */
 	BIGNUM *val[TABLE_SIZE];
+	int ret = 0;
 
 	if (BN_get_flags(p, BN_FLG_CONSTTIME) != 0) {
 		/* BN_FLG_CONSTTIME only supported by BN_mod_exp_mont() */
@@ -188,10 +193,15 @@ BN_mod_exp_simple(BIGNUM *r, const BIGNUM *a, const BIGNUM *p, const BIGNUM *m,
 		return -1;
 	}
 
+	if (r == m) {
+		BNerror(BN_R_INVALID_ARGUMENT);
+		return 0;
+	}
+
 	bits = BN_num_bits(p);
 	if (bits == 0) {
 		/* x**0 mod 1 is still zero. */
-		if (BN_is_one(m)) {
+		if (BN_abs_is_word(m, 1)) {
 			ret = 1;
 			BN_zero(r);
 		} else
@@ -202,21 +212,24 @@ BN_mod_exp_simple(BIGNUM *r, const BIGNUM *a, const BIGNUM *p, const BIGNUM *m,
 	BN_CTX_start(ctx);
 	if ((d = BN_CTX_get(ctx)) == NULL)
 		goto err;
+	if ((q = BN_CTX_get(ctx)) == NULL)
+		goto err;
 	if ((val[0] = BN_CTX_get(ctx)) == NULL)
 		goto err;
 
-	if (!BN_nnmod(val[0],a,m,ctx))
-		goto err;		/* 1 */
+	if (!BN_nnmod(val[0], a, m, ctx))
+		goto err;
 	if (BN_is_zero(val[0])) {
 		BN_zero(r);
-		ret = 1;
-		goto err;
+		goto done;
 	}
+	if (!bn_copy(q, p))
+		goto err;
 
 	window = BN_window_bits_for_exponent_size(bits);
 	if (window > 1) {
 		if (!BN_mod_mul(d, val[0], val[0], m, ctx))
-			goto err;				/* 2 */
+			goto err;
 		j = 1 << (window - 1);
 		for (i = 1; i < j; i++) {
 			if (((val[i] = BN_CTX_get(ctx)) == NULL) ||
@@ -236,7 +249,7 @@ BN_mod_exp_simple(BIGNUM *r, const BIGNUM *a, const BIGNUM *p, const BIGNUM *m,
 		goto err;
 
 	for (;;) {
-		if (BN_is_bit_set(p, wstart) == 0) {
+		if (BN_is_bit_set(q, wstart) == 0) {
 			if (!start)
 				if (!BN_mod_mul(r, r, r, m, ctx))
 					goto err;
@@ -255,7 +268,7 @@ BN_mod_exp_simple(BIGNUM *r, const BIGNUM *a, const BIGNUM *p, const BIGNUM *m,
 		for (i = 1; i < window; i++) {
 			if (wstart - i < 0)
 				break;
-			if (BN_is_bit_set(p, wstart - i)) {
+			if (BN_is_bit_set(q, wstart - i)) {
 				wvalue <<= (i - wend);
 				wvalue |= 1;
 				wend = i;
@@ -282,11 +295,14 @@ BN_mod_exp_simple(BIGNUM *r, const BIGNUM *a, const BIGNUM *p, const BIGNUM *m,
 		if (wstart < 0)
 			break;
 	}
+
+ done:
 	ret = 1;
 
-err:
+ err:
 	BN_CTX_end(ctx);
-	return (ret);
+
+	return ret;
 }
 
 /* BN_mod_exp_mont_consttime() stores the precomputed powers in a specific layout
@@ -399,7 +415,7 @@ BN_mod_exp_mont_consttime(BIGNUM *rr, const BIGNUM *a, const BIGNUM *p,
 	bits = BN_num_bits(p);
 	if (bits == 0) {
 		/* x**0 mod 1 is still zero. */
-		if (BN_is_one(m)) {
+		if (BN_abs_is_word(m, 1)) {
 			ret = 1;
 			BN_zero(rr);
 		} else
@@ -409,17 +425,10 @@ BN_mod_exp_mont_consttime(BIGNUM *rr, const BIGNUM *a, const BIGNUM *p,
 
 	BN_CTX_start(ctx);
 
-	/* Allocate a montgomery context if it was not supplied by the caller.
-	 * If this is not done, things will break in the montgomery part.
- 	 */
-	if (in_mont != NULL)
-		mont = in_mont;
-	else {
-		if ((mont = BN_MONT_CTX_new()) == NULL)
-			goto err;
-		if (!BN_MONT_CTX_set(mont, m, ctx))
-			goto err;
-	}
+	if ((mont = in_mont) == NULL)
+		mont = BN_MONT_CTX_create(m, ctx);
+	if (mont == NULL)
+		goto err;
 
 	/* Get the window size to use with size of p. */
 	window = BN_window_bits_for_ctime_exponent_size(bits);
@@ -619,15 +628,18 @@ BN_mod_exp_mont_consttime(BIGNUM *rr, const BIGNUM *a, const BIGNUM *p,
 	/* Convert the final result from montgomery to standard format */
 	if (!BN_from_montgomery(rr, &tmp, mont, ctx))
 		goto err;
+
 	ret = 1;
 
-err:
-	if ((in_mont == NULL) && (mont != NULL))
+ err:
+	if (mont != in_mont)
 		BN_MONT_CTX_free(mont);
-	freezero(powerbufFree, powerbufLen + MOD_EXP_CTIME_MIN_CACHE_LINE_WIDTH);
 	BN_CTX_end(ctx);
-	return (ret);
+	freezero(powerbufFree, powerbufLen + MOD_EXP_CTIME_MIN_CACHE_LINE_WIDTH);
+
+	return ret;
 }
+LCRYPTO_ALIAS(BN_mod_exp_mont_consttime);
 
 static int
 BN_mod_exp_mont_internal(BIGNUM *rr, const BIGNUM *a, const BIGNUM *p, const BIGNUM *m,
@@ -654,7 +666,7 @@ BN_mod_exp_mont_internal(BIGNUM *rr, const BIGNUM *a, const BIGNUM *p, const BIG
 	bits = BN_num_bits(p);
 	if (bits == 0) {
 		/* x**0 mod 1 is still zero. */
-		if (BN_is_one(m)) {
+		if (BN_abs_is_word(m, 1)) {
 			ret = 1;
 			BN_zero(rr);
 		} else
@@ -670,36 +682,26 @@ BN_mod_exp_mont_internal(BIGNUM *rr, const BIGNUM *a, const BIGNUM *p, const BIG
 	if ((val[0] = BN_CTX_get(ctx)) == NULL)
 		goto err;
 
-	/* If this is not done, things will break in the montgomery
-	 * part */
+	if ((mont = in_mont) == NULL)
+		mont = BN_MONT_CTX_create(m, ctx);
+	if (mont == NULL)
+		goto err;
 
-	if (in_mont != NULL)
-		mont = in_mont;
-	else {
-		if ((mont = BN_MONT_CTX_new()) == NULL)
-			goto err;
-		if (!BN_MONT_CTX_set(mont, m, ctx))
-			goto err;
-	}
-
-	if (a->neg || BN_ucmp(a, m) >= 0) {
-		if (!BN_nnmod(val[0], a,m, ctx))
-			goto err;
-		aa = val[0];
-	} else
-		aa = a;
+	if (!BN_nnmod(val[0], a,m, ctx))
+		goto err;
+	aa = val[0];
 	if (BN_is_zero(aa)) {
 		BN_zero(rr);
 		ret = 1;
 		goto err;
 	}
 	if (!BN_to_montgomery(val[0], aa, mont, ctx))
-		goto err; /* 1 */
+		goto err;
 
 	window = BN_window_bits_for_exponent_size(bits);
 	if (window > 1) {
 		if (!BN_mod_mul_montgomery(d, val[0], val[0], mont, ctx))
-			goto err; /* 2 */
+			goto err;
 		j = 1 << (window - 1);
 		for (i = 1; i < j; i++) {
 			if (((val[i] = BN_CTX_get(ctx)) == NULL) ||
@@ -768,13 +770,15 @@ BN_mod_exp_mont_internal(BIGNUM *rr, const BIGNUM *a, const BIGNUM *p, const BIG
 	}
 	if (!BN_from_montgomery(rr, r,mont, ctx))
 		goto err;
+
 	ret = 1;
 
-err:
-	if ((in_mont == NULL) && (mont != NULL))
+ err:
+	if (mont != in_mont)
 		BN_MONT_CTX_free(mont);
 	BN_CTX_end(ctx);
-	return (ret);
+
+	return ret;
 }
 
 int
@@ -784,6 +788,7 @@ BN_mod_exp_mont(BIGNUM *rr, const BIGNUM *a, const BIGNUM *p, const BIGNUM *m,
 	return BN_mod_exp_mont_internal(rr, a, p, m, ctx, in_mont,
 	    (BN_get_flags(p, BN_FLG_CONSTTIME) != 0));
 }
+LCRYPTO_ALIAS(BN_mod_exp_mont);
 
 int
 BN_mod_exp_mont_ct(BIGNUM *rr, const BIGNUM *a, const BIGNUM *p, const BIGNUM *m,
@@ -816,7 +821,7 @@ BN_mod_exp_mont_word(BIGNUM *rr, BN_ULONG a, const BIGNUM *p, const BIGNUM *m,
 			(BN_mod_ct(t, r, m, ctx) && (swap_tmp = r, r = t, t = swap_tmp, 1))))
 		/* BN_MOD_MUL_WORD is only used with 'w' large,
 		 * so the BN_ucmp test is probably more overhead
-		 * than always using BN_mod (which uses BN_copy if
+		 * than always using BN_mod (which uses bn_copy if
 		 * a similar test returns true). */
 		/* We can use BN_mod and do not need BN_nnmod because our
 		 * accumulator is never negative (the result of BN_mod does
@@ -842,7 +847,7 @@ BN_mod_exp_mont_word(BIGNUM *rr, BN_ULONG a, const BIGNUM *p, const BIGNUM *m,
 	bits = BN_num_bits(p);
 	if (bits == 0) {
 		/* x**0 mod 1 is still zero. */
-		if (BN_is_one(m)) {
+		if (BN_abs_is_word(m, 1)) {
 			ret = 1;
 			BN_zero(rr);
 		} else
@@ -863,14 +868,10 @@ BN_mod_exp_mont_word(BIGNUM *rr, BN_ULONG a, const BIGNUM *p, const BIGNUM *m,
 	if ((t = BN_CTX_get(ctx)) == NULL)
 		goto err;
 
-	if (in_mont != NULL)
-		mont = in_mont;
-	else {
-		if ((mont = BN_MONT_CTX_new()) == NULL)
-			goto err;
-		if (!BN_MONT_CTX_set(mont, m, ctx))
-			goto err;
-	}
+	if ((mont = in_mont) == NULL)
+		mont = BN_MONT_CTX_create(m, ctx);
+	if (mont == NULL)
+		goto err;
 
 	r_is_one = 1; /* except for Montgomery factor */
 
@@ -938,25 +939,28 @@ BN_mod_exp_mont_word(BIGNUM *rr, BN_ULONG a, const BIGNUM *p, const BIGNUM *m,
 		if (!BN_from_montgomery(rr, r, mont, ctx))
 			goto err;
 	}
+
 	ret = 1;
 
-err:
-	if ((in_mont == NULL) && (mont != NULL))
+ err:
+	if (mont != in_mont)
 		BN_MONT_CTX_free(mont);
 	BN_CTX_end(ctx);
-	return (ret);
+
+	return ret;
 }
 
 int
-BN_mod_exp_recp(BIGNUM *r, const BIGNUM *a, const BIGNUM *p, const BIGNUM *m,
+BN_mod_exp_reciprocal(BIGNUM *r, const BIGNUM *a, const BIGNUM *p, const BIGNUM *m,
     BN_CTX *ctx)
 {
-	int i, j, bits, ret = 0, wstart, wend, window, wvalue;
+	int i, j, bits, wstart, wend, window, wvalue;
 	int start = 1;
-	BIGNUM *aa;
+	BIGNUM *aa, *q;
 	/* Table of variables obtained from 'ctx' */
 	BIGNUM *val[TABLE_SIZE];
-	BN_RECP_CTX recp;
+	BN_RECP_CTX *recp = NULL;
+	int ret = 0;
 
 	if (BN_get_flags(p, BN_FLG_CONSTTIME) != 0) {
 		/* BN_FLG_CONSTTIME only supported by BN_mod_exp_mont() */
@@ -967,7 +971,7 @@ BN_mod_exp_recp(BIGNUM *r, const BIGNUM *a, const BIGNUM *p, const BIGNUM *m,
 	bits = BN_num_bits(p);
 	if (bits == 0) {
 		/* x**0 mod 1 is still zero. */
-		if (BN_is_one(m)) {
+		if (BN_abs_is_word(m, 1)) {
 			ret = 1;
 			BN_zero(r);
 		} else
@@ -975,43 +979,35 @@ BN_mod_exp_recp(BIGNUM *r, const BIGNUM *a, const BIGNUM *p, const BIGNUM *m,
 		return ret;
 	}
 
-	BN_RECP_CTX_init(&recp);
-
 	BN_CTX_start(ctx);
 	if ((aa = BN_CTX_get(ctx)) == NULL)
 		goto err;
+	if ((q = BN_CTX_get(ctx)) == NULL)
+		goto err;
 	if ((val[0] = BN_CTX_get(ctx)) == NULL)
 		goto err;
 
-	if (m->neg) {
-		/* ignore sign of 'm' */
-		if (!BN_copy(aa, m))
-			goto err;
-		aa->neg = 0;
-		if (BN_RECP_CTX_set(&recp, aa, ctx) <= 0)
-			goto err;
-	} else {
-		if (BN_RECP_CTX_set(&recp, m, ctx) <= 0)
-			goto err;
-	}
+	if ((recp = BN_RECP_CTX_create(m)) == NULL)
+		goto err;
 
 	if (!BN_nnmod(val[0], a, m, ctx))
-		goto err;		/* 1 */
+		goto err;
 	if (BN_is_zero(val[0])) {
 		BN_zero(r);
-		ret = 1;
-		goto err;
+		goto done;
 	}
+	if (!bn_copy(q, p))
+		goto err;
 
 	window = BN_window_bits_for_exponent_size(bits);
 	if (window > 1) {
-		if (!BN_mod_mul_reciprocal(aa, val[0], val[0], &recp, ctx))
-			goto err;				/* 2 */
+		if (!BN_mod_sqr_reciprocal(aa, val[0], recp, ctx))
+			goto err;
 		j = 1 << (window - 1);
 		for (i = 1; i < j; i++) {
 			if (((val[i] = BN_CTX_get(ctx)) == NULL) ||
 			    !BN_mod_mul_reciprocal(val[i], val[i - 1],
-			    aa, &recp, ctx))
+			    aa, recp, ctx))
 				goto err;
 		}
 	}
@@ -1027,9 +1023,9 @@ BN_mod_exp_recp(BIGNUM *r, const BIGNUM *a, const BIGNUM *p, const BIGNUM *m,
 		goto err;
 
 	for (;;) {
-		if (BN_is_bit_set(p, wstart) == 0) {
+		if (BN_is_bit_set(q, wstart) == 0) {
 			if (!start)
-				if (!BN_mod_mul_reciprocal(r, r,r, &recp, ctx))
+				if (!BN_mod_sqr_reciprocal(r, r, recp, ctx))
 					goto err;
 			if (wstart == 0)
 				break;
@@ -1046,7 +1042,7 @@ BN_mod_exp_recp(BIGNUM *r, const BIGNUM *a, const BIGNUM *p, const BIGNUM *m,
 		for (i = 1; i < window; i++) {
 			if (wstart - i < 0)
 				break;
-			if (BN_is_bit_set(p, wstart - i)) {
+			if (BN_is_bit_set(q, wstart - i)) {
 				wvalue <<= (i - wend);
 				wvalue |= 1;
 				wend = i;
@@ -1058,12 +1054,12 @@ BN_mod_exp_recp(BIGNUM *r, const BIGNUM *a, const BIGNUM *p, const BIGNUM *m,
 		/* add the 'bytes above' */
 		if (!start)
 			for (i = 0; i < j; i++) {
-				if (!BN_mod_mul_reciprocal(r, r,r, &recp, ctx))
+				if (!BN_mod_sqr_reciprocal(r, r, recp, ctx))
 					goto err;
 			}
 
 		/* wvalue will be an odd number < 2^window */
-		if (!BN_mod_mul_reciprocal(r, r,val[wvalue >> 1], &recp, ctx))
+		if (!BN_mod_mul_reciprocal(r, r, val[wvalue >> 1], recp, ctx))
 			goto err;
 
 		/* move the 'window' down further */
@@ -1073,12 +1069,15 @@ BN_mod_exp_recp(BIGNUM *r, const BIGNUM *a, const BIGNUM *p, const BIGNUM *m,
 		if (wstart < 0)
 			break;
 	}
+
+ done:
 	ret = 1;
 
-err:
+ err:
 	BN_CTX_end(ctx);
-	BN_RECP_CTX_free(&recp);
-	return (ret);
+	BN_RECP_CTX_free(recp);
+
+	return ret;
 }
 
 static int
@@ -1103,7 +1102,7 @@ BN_mod_exp_internal(BIGNUM *r, const BIGNUM *a, const BIGNUM *p, const BIGNUM *m
 	 * standard algorithm:
 	 *
 	 *   BN_mod_exp_mont   33 .. 40 %  [AMD K6-2, Linux, debug configuration]
-         *                     55 .. 77 %  [UltraSparc processor, but
+	 *                     55 .. 77 %  [UltraSparc processor, but
 	 *                                  debug-solaris-sparcv8-gcc conf.]
 	 *
 	 *   BN_mod_exp_recp   50 .. 70 %  [AMD K6-2, Linux, debug configuration]
@@ -1125,7 +1124,7 @@ BN_mod_exp_internal(BIGNUM *r, const BIGNUM *a, const BIGNUM *p, const BIGNUM *m
 		} else
 			ret = BN_mod_exp_mont_ct(r, a,p, m,ctx, NULL);
 	} else	{
-		ret = BN_mod_exp_recp(r, a,p, m, ctx);
+		ret = BN_mod_exp_reciprocal(r, a,p, m, ctx);
 	}
 
 	return (ret);
@@ -1138,6 +1137,7 @@ BN_mod_exp(BIGNUM *r, const BIGNUM *a, const BIGNUM *p, const BIGNUM *m,
 	return BN_mod_exp_internal(r, a, p, m, ctx,
 	    (BN_get_flags(p, BN_FLG_CONSTTIME) != 0));
 }
+LCRYPTO_ALIAS(BN_mod_exp);
 
 int
 BN_mod_exp_ct(BIGNUM *r, const BIGNUM *a, const BIGNUM *p, const BIGNUM *m,
@@ -1190,14 +1190,10 @@ BN_mod_exp2_mont(BIGNUM *rr, const BIGNUM *a1, const BIGNUM *p1,
 	if ((val2[0] = BN_CTX_get(ctx)) == NULL)
 		goto err;
 
-	if (in_mont != NULL)
-		mont = in_mont;
-	else {
-		if ((mont = BN_MONT_CTX_new()) == NULL)
-			goto err;
-		if (!BN_MONT_CTX_set(mont, m, ctx))
-			goto err;
-	}
+	if ((mont = in_mont) == NULL)
+		mont = BN_MONT_CTX_create(m, ctx);
+	if (mont == NULL)
+		goto err;
 
 	window1 = BN_window_bits_for_exponent_size(bits1);
 	window2 = BN_window_bits_for_exponent_size(bits2);
@@ -1205,12 +1201,9 @@ BN_mod_exp2_mont(BIGNUM *rr, const BIGNUM *a1, const BIGNUM *p1,
 	/*
 	 * Build table for a1:   val1[i] := a1^(2*i + 1) mod m  for i = 0 .. 2^(window1-1)
 	 */
-	if (a1->neg || BN_ucmp(a1, m) >= 0) {
-		if (!BN_mod_ct(val1[0], a1, m, ctx))
-			goto err;
-		a_mod_m = val1[0];
-	} else
-		a_mod_m = a1;
+	if (!BN_nnmod(val1[0], a1, m, ctx))
+		goto err;
+	a_mod_m = val1[0];
 	if (BN_is_zero(a_mod_m)) {
 		BN_zero(rr);
 		ret = 1;
@@ -1236,12 +1229,9 @@ BN_mod_exp2_mont(BIGNUM *rr, const BIGNUM *a1, const BIGNUM *p1,
 	/*
 	 * Build table for a2:   val2[i] := a2^(2*i + 1) mod m  for i = 0 .. 2^(window2-1)
 	 */
-	if (a2->neg || BN_ucmp(a2, m) >= 0) {
-		if (!BN_mod_ct(val2[0], a2, m, ctx))
-			goto err;
-		a_mod_m = val2[0];
-	} else
-		a_mod_m = a2;
+	if (!BN_nnmod(val2[0], a2, m, ctx))
+		goto err;
+	a_mod_m = val2[0];
 	if (BN_is_zero(a_mod_m)) {
 		BN_zero(rr);
 		ret = 1;
@@ -1328,11 +1318,13 @@ BN_mod_exp2_mont(BIGNUM *rr, const BIGNUM *a1, const BIGNUM *p1,
 	}
 	if (!BN_from_montgomery(rr, r,mont, ctx))
 		goto err;
+
 	ret = 1;
 
-err:
-	if ((in_mont == NULL) && (mont != NULL))
+ err:
+	if (mont != in_mont)
 		BN_MONT_CTX_free(mont);
 	BN_CTX_end(ctx);
-	return (ret);
+
+	return ret;
 }
