@@ -1,4 +1,4 @@
-/* $OpenBSD: bn_mont.c,v 1.63 2024/03/26 04:23:04 jsing Exp $ */
+/* $OpenBSD: bn_mont.c,v 1.66 2025/03/09 15:22:40 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -154,7 +154,25 @@ BN_MONT_CTX_free(BN_MONT_CTX *mctx)
 LCRYPTO_ALIAS(BN_MONT_CTX_free);
 
 BN_MONT_CTX *
-BN_MONT_CTX_copy(BN_MONT_CTX *dst, BN_MONT_CTX *src)
+BN_MONT_CTX_create(const BIGNUM *bn, BN_CTX *bn_ctx)
+{
+	BN_MONT_CTX *mctx;
+
+	if ((mctx = BN_MONT_CTX_new()) == NULL)
+		goto err;
+	if (!BN_MONT_CTX_set(mctx, bn, bn_ctx))
+		goto err;
+
+	return mctx;
+
+ err:
+	BN_MONT_CTX_free(mctx);
+
+	return NULL;
+}
+
+BN_MONT_CTX *
+BN_MONT_CTX_copy(BN_MONT_CTX *dst, const BN_MONT_CTX *src)
 {
 	if (dst == src)
 		return dst;
@@ -275,9 +293,7 @@ BN_MONT_CTX_set_locked(BN_MONT_CTX **pmctx, int lock, const BIGNUM *mod,
 	if (mctx != NULL)
 		goto done;
 
-	if ((mctx = BN_MONT_CTX_new()) == NULL)
-		goto err;
-	if (!BN_MONT_CTX_set(mctx, mod, ctx))
+	if ((mctx = BN_MONT_CTX_create(mod, ctx)) == NULL)
 		goto err;
 
 	CRYPTO_w_lock(lock);
