@@ -1,4 +1,4 @@
-/* $OpenBSD: bn_rand.c,v 1.24 2020/09/12 17:16:36 tb Exp $ */
+/* $OpenBSD: bn_rand.c,v 1.30 2024/03/16 20:42:33 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -109,6 +109,7 @@
  *
  */
 
+#include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -116,7 +117,7 @@
 
 #include <openssl/err.h>
 
-#include "bn_lcl.h"
+#include "bn_local.h"
 
 static int
 bnrand(int pseudorand, BIGNUM *rnd, int bits, int top, int bottom)
@@ -133,6 +134,10 @@ bnrand(int pseudorand, BIGNUM *rnd, int bits, int top, int bottom)
 		BNerror(BN_R_BITS_TOO_SMALL);
 		return (0);
 	}
+	if (bits > INT_MAX - 7) {
+		BNerror(BN_R_BIGNUM_TOO_LONG);
+		return (0);
+	}
 
 	if (bits == 0) {
 		BN_zero(rnd);
@@ -190,24 +195,25 @@ bnrand(int pseudorand, BIGNUM *rnd, int bits, int top, int bottom)
 
 err:
 	freezero(buf, bytes);
-	bn_check_top(rnd);
 	return (ret);
 }
 
-int    
+int
 BN_rand(BIGNUM *rnd, int bits, int top, int bottom)
 {
 	return bnrand(0, rnd, bits, top, bottom);
 }
+LCRYPTO_ALIAS(BN_rand);
 
-int    
+int
 BN_pseudo_rand(BIGNUM *rnd, int bits, int top, int bottom)
 {
 	return bnrand(1, rnd, bits, top, bottom);
 }
+LCRYPTO_ALIAS(BN_pseudo_rand);
 
 #if 1
-int    
+int
 BN_bntest_rand(BIGNUM *rnd, int bits, int top, int bottom)
 {
 	return bnrand(2, rnd, bits, top, bottom);
@@ -272,7 +278,6 @@ bn_rand_range(int pseudo, BIGNUM *r, const BIGNUM *range)
 		} while (BN_cmp(r, range) >= 0);
 	}
 
-	bn_check_top(r);
 	return 1;
 }
 
@@ -281,31 +286,49 @@ BN_rand_range(BIGNUM *r, const BIGNUM *range)
 {
 	return bn_rand_range(0, r, range);
 }
+LCRYPTO_ALIAS(BN_rand_range);
 
 int
-bn_rand_interval(BIGNUM *rnd, const BIGNUM *lower_inc, const BIGNUM *upper_exc)
+bn_rand_in_range(BIGNUM *rnd, const BIGNUM *lower_inc, const BIGNUM *upper_exc)
 {
-	BIGNUM *len = NULL;
+	BIGNUM *len;
 	int ret = 0;
 
-	if (BN_cmp(lower_inc, upper_exc) >= 0)
-		goto err;
-
 	if ((len = BN_new()) == NULL)
 		goto err;
-
 	if (!BN_sub(len, upper_exc, lower_inc))
 		goto err;
-
-	if (!bn_rand_range(0, rnd, len))
+	if (!BN_rand_range(rnd, len))
 		goto err;
-
 	if (!BN_add(rnd, rnd, lower_inc))
 		goto err;
 
 	ret = 1;
+
  err:
 	BN_free(len);
+
+	return ret;
+}
+
+int
+bn_rand_interval(BIGNUM *rnd, BN_ULONG lower_word, const BIGNUM *upper_exc)
+{
+	BIGNUM *lower_inc = NULL;
+	int ret = 0;
+
+	if ((lower_inc = BN_new()) == NULL)
+		goto err;
+	if (!BN_set_word(lower_inc, lower_word))
+		goto err;
+	if (!bn_rand_in_range(rnd, lower_inc, upper_exc))
+		goto err;
+
+	ret = 1;
+
+ err:
+	BN_free(lower_inc);
+
 	return ret;
 }
 
@@ -314,3 +337,4 @@ BN_pseudo_rand_range(BIGNUM *r, const BIGNUM *range)
 {
 	return bn_rand_range(1, r, range);
 }
+LCRYPTO_ALIAS(BN_pseudo_rand_range);
