@@ -1,4 +1,4 @@
-/* $OpenBSD: e_des.c,v 1.23 2024/01/04 17:38:36 tb Exp $ */
+/* $OpenBSD: e_des.c,v 1.24 2024/04/09 13:52:41 beck Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -241,6 +241,7 @@ EVP_des_cbc(void)
 {
 	return &des_cbc;
 }
+LCRYPTO_ALIAS(EVP_des_cbc);
 
 static const EVP_CIPHER des_cfb64 = {
 	.nid = NID_des_cfb64,
@@ -262,6 +263,7 @@ EVP_des_cfb64(void)
 {
 	return &des_cfb64;
 }
+LCRYPTO_ALIAS(EVP_des_cfb64);
 
 static const EVP_CIPHER des_ofb = {
 	.nid = NID_des_ofb64,
@@ -283,6 +285,7 @@ EVP_des_ofb(void)
 {
 	return &des_ofb;
 }
+LCRYPTO_ALIAS(EVP_des_ofb);
 
 static const EVP_CIPHER des_ecb = {
 	.nid = NID_des_ecb,
@@ -304,6 +307,7 @@ EVP_des_ecb(void)
 {
 	return &des_ecb;
 }
+LCRYPTO_ALIAS(EVP_des_ecb);
 
 static const EVP_CIPHER des_cfb1 = {
 	.nid = NID_des_cfb1,
@@ -325,6 +329,7 @@ EVP_des_cfb1(void)
 {
 	return &des_cfb1;
 }
+LCRYPTO_ALIAS(EVP_des_cfb1);
 
 static const EVP_CIPHER des_cfb8 = {
 	.nid = NID_des_cfb8,
@@ -346,4 +351,5 @@ EVP_des_cfb8(void)
 {
 	return &des_cfb8;
 }
+LCRYPTO_ALIAS(EVP_des_cfb8);
 #endif
