@@ -1,4 +1,4 @@
-/* $OpenBSD: evp_err.c,v 1.28 2022/07/12 14:42:49 kn Exp $ */
+/* $OpenBSD: evp_err.c,v 1.34 2024/06/24 06:43:22 tb Exp $ */
 /* ====================================================================
  * Copyright (c) 1999-2011 The OpenSSL Project.  All rights reserved.
  *
@@ -60,17 +60,19 @@
 #include <openssl/err.h>
 #include <openssl/evp.h>
 
+#include "err_local.h"
+
 #ifndef OPENSSL_NO_ERR
 
 #define ERR_FUNC(func) ERR_PACK(ERR_LIB_EVP,func,0)
 #define ERR_REASON(reason) ERR_PACK(ERR_LIB_EVP,0,reason)
 
-static ERR_STRING_DATA EVP_str_functs[] = {
+static const ERR_STRING_DATA EVP_str_functs[] = {
 	{ERR_FUNC(0xfff), "CRYPTO_internal"},
 	{0, NULL}
 };
 
-static ERR_STRING_DATA EVP_str_reasons[] = {
+static const ERR_STRING_DATA EVP_str_reasons[] = {
 	{ERR_REASON(EVP_R_AES_IV_SETUP_FAILED)   , "aes iv setup failed"},
 	{ERR_REASON(EVP_R_AES_KEY_SETUP_FAILED)  , "aes key setup failed"},
 	{ERR_REASON(EVP_R_ASN1_LIB)              , "asn1 lib"},
@@ -101,6 +103,7 @@ static ERR_STRING_DATA EVP_str_reasons[] = {
 	{ERR_REASON(EVP_R_EXPECTING_A_ECDSA_KEY) , "expecting a ecdsa key"},
 	{ERR_REASON(EVP_R_EXPECTING_A_EC_KEY)    , "expecting a ec key"},
 	{ERR_REASON(EVP_R_FIPS_MODE_NOT_SUPPORTED), "fips mode not supported"},
+	{ERR_REASON(EVP_R_GET_RAW_KEY_FAILED)    , "get raw key failed"},
 	{ERR_REASON(EVP_R_INITIALIZATION_ERROR)  , "initialization error"},
 	{ERR_REASON(EVP_R_INPUT_NOT_INITIALIZED) , "input not initialized"},
 	{ERR_REASON(EVP_R_INVALID_DIGEST)        , "invalid digest"},
@@ -122,6 +125,7 @@ static ERR_STRING_DATA EVP_str_reasons[] = {
 	{ERR_REASON(EVP_R_NO_OPERATION_SET)      , "no operation set"},
 	{ERR_REASON(EVP_R_NO_SIGN_FUNCTION_CONFIGURED), "no sign function configured"},
 	{ERR_REASON(EVP_R_NO_VERIFY_FUNCTION_CONFIGURED), "no verify function configured"},
+	{ERR_REASON(EVP_R_ONLY_ONESHOT_SUPPORTED), "only oneshot supported"},
 	{ERR_REASON(EVP_R_OPERATION_NOT_SUPPORTED_FOR_THIS_KEYTYPE), "operation not supported for this keytype"},
 	{ERR_REASON(EVP_R_OPERATON_NOT_INITIALIZED), "operaton not initialized"},
 	{ERR_REASON(EVP_R_OUTPUT_ALIASES_INPUT)  , "output aliases input"},
@@ -157,8 +161,9 @@ ERR_load_EVP_strings(void)
 {
 #ifndef OPENSSL_NO_ERR
 	if (ERR_func_error_string(EVP_str_functs[0].error) == NULL) {
-		ERR_load_strings(0, EVP_str_functs);
-		ERR_load_strings(0, EVP_str_reasons);
+		ERR_load_const_strings(EVP_str_functs);
+		ERR_load_const_strings(EVP_str_reasons);
 	}
 #endif
 }
+LCRYPTO_ALIAS(ERR_load_EVP_strings);
