@@ -1,4 +1,4 @@
-/* $OpenBSD: ts_req_utils.c,v 1.6 2018/05/13 15:04:05 tb Exp $ */
+/* $OpenBSD: ts_req_utils.c,v 1.9 2023/07/07 19:37:54 beck Exp $ */
 /* Written by Zoltan Glozik (zglozik@stones.com) for the OpenSSL
  * project 2002.
  */
@@ -63,17 +63,21 @@
 #include <openssl/ts.h>
 #include <openssl/x509v3.h>
 
+#include "ts_local.h"
+
 int
 TS_REQ_set_version(TS_REQ *a, long version)
 {
 	return ASN1_INTEGER_set(a->version, version);
 }
+LCRYPTO_ALIAS(TS_REQ_set_version);
 
 long
 TS_REQ_get_version(const TS_REQ *a)
 {
 	return ASN1_INTEGER_get(a->version);
 }
+LCRYPTO_ALIAS(TS_REQ_get_version);
 
 int
 TS_REQ_set_msg_imprint(TS_REQ *a, TS_MSG_IMPRINT *msg_imprint)
@@ -91,12 +95,14 @@ TS_REQ_set_msg_imprint(TS_REQ *a, TS_MSG_IMPRINT *msg_imprint)
 	a->msg_imprint = new_msg_imprint;
 	return 1;
 }
+LCRYPTO_ALIAS(TS_REQ_set_msg_imprint);
 
 TS_MSG_IMPRINT *
 TS_REQ_get_msg_imprint(TS_REQ *a)
 {
 	return a->msg_imprint;
 }
+LCRYPTO_ALIAS(TS_REQ_get_msg_imprint);
 
 int
 TS_MSG_IMPRINT_set_algo(TS_MSG_IMPRINT *a, X509_ALGOR *alg)
@@ -114,24 +120,28 @@ TS_MSG_IMPRINT_set_algo(TS_MSG_IMPRINT *a, X509_ALGOR *alg)
 	a->hash_algo = new_alg;
 	return 1;
 }
+LCRYPTO_ALIAS(TS_MSG_IMPRINT_set_algo);
 
 X509_ALGOR *
 TS_MSG_IMPRINT_get_algo(TS_MSG_IMPRINT *a)
 {
 	return a->hash_algo;
 }
+LCRYPTO_ALIAS(TS_MSG_IMPRINT_get_algo);
 
 int
 TS_MSG_IMPRINT_set_msg(TS_MSG_IMPRINT *a, unsigned char *d, int len)
 {
 	return ASN1_OCTET_STRING_set(a->hashed_msg, d, len);
 }
+LCRYPTO_ALIAS(TS_MSG_IMPRINT_set_msg);
 
 ASN1_OCTET_STRING *
 TS_MSG_IMPRINT_get_msg(TS_MSG_IMPRINT *a)
 {
 	return a->hashed_msg;
 }
+LCRYPTO_ALIAS(TS_MSG_IMPRINT_get_msg);
 
 int
 TS_REQ_set_policy_id(TS_REQ *a, const ASN1_OBJECT *policy)
@@ -149,12 +159,14 @@ TS_REQ_set_policy_id(TS_REQ *a, const ASN1_OBJECT *policy)
 	a->policy_id = new_policy;
 	return 1;
 }
+LCRYPTO_ALIAS(TS_REQ_set_policy_id);
 
 ASN1_OBJECT *
 TS_REQ_get_policy_id(TS_REQ *a)
 {
 	return a->policy_id;
 }
+LCRYPTO_ALIAS(TS_REQ_get_policy_id);
 
 int
 TS_REQ_set_nonce(TS_REQ *a, const ASN1_INTEGER *nonce)
@@ -172,12 +184,14 @@ TS_REQ_set_nonce(TS_REQ *a, const ASN1_INTEGER *nonce)
 	a->nonce = new_nonce;
 	return 1;
 }
+LCRYPTO_ALIAS(TS_REQ_set_nonce);
 
 const ASN1_INTEGER *
 TS_REQ_get_nonce(const TS_REQ *a)
 {
 	return a->nonce;
 }
+LCRYPTO_ALIAS(TS_REQ_get_nonce);
 
 int
 TS_REQ_set_cert_req(TS_REQ *a, int cert_req)
@@ -185,17 +199,20 @@ TS_REQ_set_cert_req(TS_REQ *a, int cert_req)
 	a->cert_req = cert_req ? 0xFF : 0x00;
 	return 1;
 }
+LCRYPTO_ALIAS(TS_REQ_set_cert_req);
 
 int
 TS_REQ_get_cert_req(const TS_REQ *a)
 {
 	return a->cert_req ? 1 : 0;
 }
+LCRYPTO_ALIAS(TS_REQ_get_cert_req);
 
 STACK_OF(X509_EXTENSION) *TS_REQ_get_exts(TS_REQ *a)
 {
 	return a->extensions;
 }
+LCRYPTO_ALIAS(TS_REQ_get_exts);
 
 void
 TS_REQ_ext_free(TS_REQ *a)
@@ -205,51 +222,60 @@ TS_REQ_ext_free(TS_REQ *a)
 	sk_X509_EXTENSION_pop_free(a->extensions, X509_EXTENSION_free);
 	a->extensions = NULL;
 }
+LCRYPTO_ALIAS(TS_REQ_ext_free);
 
 int
 TS_REQ_get_ext_count(TS_REQ *a)
 {
 	return X509v3_get_ext_count(a->extensions);
 }
+LCRYPTO_ALIAS(TS_REQ_get_ext_count);
 
 int
 TS_REQ_get_ext_by_NID(TS_REQ *a, int nid, int lastpos)
 {
 	return X509v3_get_ext_by_NID(a->extensions, nid, lastpos);
 }
+LCRYPTO_ALIAS(TS_REQ_get_ext_by_NID);
 
 int
 TS_REQ_get_ext_by_OBJ(TS_REQ *a, const ASN1_OBJECT *obj, int lastpos)
 {
 	return X509v3_get_ext_by_OBJ(a->extensions, obj, lastpos);
 }
+LCRYPTO_ALIAS(TS_REQ_get_ext_by_OBJ);
 
 int
 TS_REQ_get_ext_by_critical(TS_REQ *a, int crit, int lastpos)
 {
 	return X509v3_get_ext_by_critical(a->extensions, crit, lastpos);
 }
+LCRYPTO_ALIAS(TS_REQ_get_ext_by_critical);
 
 X509_EXTENSION *
 TS_REQ_get_ext(TS_REQ *a, int loc)
 {
 	return X509v3_get_ext(a->extensions, loc);
 }
+LCRYPTO_ALIAS(TS_REQ_get_ext);
 
 X509_EXTENSION *
 TS_REQ_delete_ext(TS_REQ *a, int loc)
 {
 	return X509v3_delete_ext(a->extensions, loc);
 }
+LCRYPTO_ALIAS(TS_REQ_delete_ext);
 
 int
 TS_REQ_add_ext(TS_REQ *a, X509_EXTENSION *ex, int loc)
 {
 	return X509v3_add_ext(&a->extensions, ex, loc) != NULL;
 }
+LCRYPTO_ALIAS(TS_REQ_add_ext);
 
 void *
 TS_REQ_get_ext_d2i(TS_REQ *a, int nid, int *crit, int *idx)
 {
 	return X509V3_get_d2i(a->extensions, nid, crit, idx);
 }
+LCRYPTO_ALIAS(TS_REQ_get_ext_d2i);
