@@ -1,4 +1,4 @@
-/* $OpenBSD: x509_req.c,v 1.32 2023/02/16 08:38:17 tb Exp $ */
+/* $OpenBSD: x509_req.c,v 1.43 2024/08/31 10:16:52 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -70,59 +70,52 @@
 #include <openssl/pem.h>
 #include <openssl/x509.h>
 
+#include "asn1_local.h"
 #include "evp_local.h"
 #include "x509_local.h"
 
 X509_REQ *
-X509_to_X509_REQ(X509 *x, EVP_PKEY *pkey, const EVP_MD *md)
+X509_to_X509_REQ(X509 *x509, EVP_PKEY *signing_key, const EVP_MD *signing_md)
 {
-	X509_REQ *ret;
-	X509_REQ_INFO *ri;
-	int i;
-	EVP_PKEY *pktmp;
+	X509_REQ *req;
+	X509_NAME *subject;
+	EVP_PKEY *public_key;
 
-	ret = X509_REQ_new();
-	if (ret == NULL) {
+	if ((req = X509_REQ_new()) == NULL) {
 		X509error(ERR_R_MALLOC_FAILURE);
 		goto err;
 	}
 
-	ri = ret->req_info;
-
-	if ((ri->version = ASN1_INTEGER_new()) == NULL)
+	if ((subject = X509_get_subject_name(x509)) == NULL)
 		goto err;
-	if (ASN1_INTEGER_set(ri->version, 0) == 0)
+	if (!X509_REQ_set_subject_name(req, subject))
 		goto err;
 
-	if (!X509_REQ_set_subject_name(ret, X509_get_subject_name(x)))
+	if ((public_key = X509_get0_pubkey(x509)) == NULL)
+		goto err;
+	if (!X509_REQ_set_pubkey(req, public_key))
 		goto err;
 
-	if ((pktmp = X509_get_pubkey(x)) == NULL)
-		goto err;
-
-	i = X509_REQ_set_pubkey(ret, pktmp);
-	EVP_PKEY_free(pktmp);
-	if (!i)
-		goto err;
-
-	if (pkey != NULL) {
-		if (!X509_REQ_sign(ret, pkey, md))
+	if (signing_key != NULL) {
+		if (!X509_REQ_sign(req, signing_key, signing_md))
 			goto err;
 	}
-	return (ret);
 
-err:
-	X509_REQ_free(ret);
-	return (NULL);
+	return req;
+
+ err:
+	X509_REQ_free(req);
+
+	return NULL;
 }
 LCRYPTO_ALIAS(X509_to_X509_REQ);
 
 EVP_PKEY *
 X509_REQ_get_pubkey(X509_REQ *req)
 {
-	if ((req == NULL) || (req->req_info == NULL))
-		return (NULL);
-	return (X509_PUBKEY_get(req->req_info->pubkey));
+	if (req == NULL || req->req_info == NULL)
+		return NULL;
+	return X509_PUBKEY_get(req->req_info->pubkey);
 }
 LCRYPTO_ALIAS(X509_REQ_get_pubkey);
 
@@ -136,107 +129,74 @@ X509_REQ_get0_pubkey(X509_REQ *req)
 LCRYPTO_ALIAS(X509_REQ_get0_pubkey);
 
 int
-X509_REQ_check_private_key(X509_REQ *x, EVP_PKEY *k)
+X509_REQ_check_private_key(X509_REQ *req, EVP_PKEY *pkey)
 {
-	EVP_PKEY *xk = NULL;
-	int ok = 0;
+	EVP_PKEY *req_pubkey = NULL;
+	int ret;
 
-	if ((xk = X509_REQ_get0_pubkey(x)) == NULL)
+	if ((req_pubkey = X509_REQ_get0_pubkey(req)) == NULL)
 		return 0;
 
-	switch (EVP_PKEY_cmp(xk, k)) {
-	case 1:
-		ok = 1;
-		break;
+	if ((ret = EVP_PKEY_cmp(req_pubkey, pkey)) == 1)
+		return 1;
+
+	switch (ret) {
 	case 0:
 		X509error(X509_R_KEY_VALUES_MISMATCH);
-		break;
+		return 0;
 	case -1:
 		X509error(X509_R_KEY_TYPE_MISMATCH);
-		break;
+		return 0;
 	case -2:
 #ifndef OPENSSL_NO_EC
-		if (k->type == EVP_PKEY_EC) {
+		if (pkey->type == EVP_PKEY_EC) {
 			X509error(ERR_R_EC_LIB);
-			break;
+			return 0;
 		}
 #endif
 #ifndef OPENSSL_NO_DH
-		if (k->type == EVP_PKEY_DH) {
+		if (pkey->type == EVP_PKEY_DH) {
 			/* No idea */
 			X509error(X509_R_CANT_CHECK_DH_KEY);
-			break;
+			return 0;
 		}
 #endif
 		X509error(X509_R_UNKNOWN_KEY_TYPE);
+		return 0;
 	}
 
-	return (ok);
+	return 0;
 }
 LCRYPTO_ALIAS(X509_REQ_check_private_key);
 
-/* It seems several organisations had the same idea of including a list of
- * extensions in a certificate request. There are at least two OIDs that are
- * used and there may be more: so the list is configurable.
- */
-
-static int ext_nid_list[] = {NID_ext_req, NID_ms_ext_req, NID_undef};
-
-static int *ext_nids = ext_nid_list;
-
 int
-X509_REQ_extension_nid(int req_nid)
+X509_REQ_extension_nid(int nid)
 {
-	int i, nid;
-
-	for (i = 0; ; i++) {
-		nid = ext_nids[i];
-		if (nid == NID_undef)
-			return 0;
-		else if (req_nid == nid)
-			return 1;
-	}
+	return nid == NID_ext_req || nid == NID_ms_ext_req;
 }
 LCRYPTO_ALIAS(X509_REQ_extension_nid);
 
-int *
-X509_REQ_get_extension_nids(void)
-{
-	return ext_nids;
-}
-LCRYPTO_ALIAS(X509_REQ_get_extension_nids);
-
-void
-X509_REQ_set_extension_nids(int *nids)
-{
-	ext_nids = nids;
-}
-LCRYPTO_ALIAS(X509_REQ_set_extension_nids);
-
 STACK_OF(X509_EXTENSION) *
 X509_REQ_get_extensions(X509_REQ *req)
 {
 	X509_ATTRIBUTE *attr;
 	ASN1_TYPE *ext = NULL;
-	int idx, *pnid;
-	const unsigned char *p;
+	int idx;
 
-	if (req == NULL || req->req_info == NULL || ext_nids == NULL)
+	if (req == NULL || req->req_info == NULL)
 		return NULL;
-	for (pnid = ext_nids; *pnid != NID_undef; pnid++) {
-		idx = X509_REQ_get_attr_by_NID(req, *pnid, -1);
-		if (idx == -1)
-			continue;
-		attr = X509_REQ_get_attr(req, idx);
-		ext = X509_ATTRIBUTE_get0_type(attr, 0);
-		break;
-	}
-	if (ext == NULL)
-		return sk_X509_EXTENSION_new_null();
-	if (ext->type != V_ASN1_SEQUENCE)
+
+	if ((idx = X509_REQ_get_attr_by_NID(req, NID_ext_req, -1)) == -1)
+		idx = X509_REQ_get_attr_by_NID(req, NID_ms_ext_req, -1);
+	if (idx == -1)
 		return NULL;
-	p = ext->value.sequence->data;
-	return d2i_X509_EXTENSIONS(NULL, &p, ext->value.sequence->length);
+
+	if ((attr = X509_REQ_get_attr(req, idx)) == NULL)
+		return NULL;
+	if ((ext = X509_ATTRIBUTE_get0_type(attr, 0)) == NULL)
+		return NULL;
+
+	return ASN1_TYPE_unpack_sequence(&X509_EXTENSIONS_it, ext);
 }
 LCRYPTO_ALIAS(X509_REQ_get_extensions);
 
@@ -251,16 +211,15 @@ X509_REQ_add_extensions_nid(X509_REQ *req, STACK_OF(X509_EXTENSION) *exts,
 {
 	unsigned char *ext = NULL;
 	int extlen;
-	int rv;
+	int ret;
 
-	extlen = i2d_X509_EXTENSIONS(exts, &ext);
-	if (extlen <= 0)
+	if ((extlen = i2d_X509_EXTENSIONS(exts, &ext)) <= 0)
 		return 0;
 
-	rv = X509_REQ_add1_attr_by_NID(req, nid, V_ASN1_SEQUENCE, ext, extlen);
+	ret = X509_REQ_add1_attr_by_NID(req, nid, V_ASN1_SEQUENCE, ext, extlen);
 	free(ext);
 
-	return rv;
+	return ret;
 }
 LCRYPTO_ALIAS(X509_REQ_add_extensions_nid);
 
@@ -277,7 +236,7 @@ LCRYPTO_ALIAS(X509_REQ_add_extensions);
 int
 X509_REQ_get_attr_count(const X509_REQ *req)
 {
-	return X509at_get_attr_count(req->req_info->attributes);
+	return sk_X509_ATTRIBUTE_num(req->req_info->attributes);
 }
 LCRYPTO_ALIAS(X509_REQ_get_attr_count);
 
@@ -299,14 +258,14 @@ LCRYPTO_ALIAS(X509_REQ_get_attr_by_OBJ);
 X509_ATTRIBUTE *
 X509_REQ_get_attr(const X509_REQ *req, int loc)
 {
-	return X509at_get_attr(req->req_info->attributes, loc);
+	return sk_X509_ATTRIBUTE_value(req->req_info->attributes, loc);
 }
 LCRYPTO_ALIAS(X509_REQ_get_attr);
 
 X509_ATTRIBUTE *
 X509_REQ_delete_attr(X509_REQ *req, int loc)
 {
-	return X509at_delete_attr(req->req_info->attributes, loc);
+	return sk_X509_ATTRIBUTE_delete(req->req_info->attributes, loc);
 }
 LCRYPTO_ALIAS(X509_REQ_delete_attr);
 
