@@ -1,4 +1,4 @@
-/*	$OpenBSD: ssltest.c,v 1.39 2023/04/15 16:50:05 tb Exp $ */
+/*	$OpenBSD: ssltest.c,v 1.45 2024/03/01 03:45:16 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -140,6 +140,9 @@
  * OTHERWISE.
  */
 
+/* XXX - USE_BIOPAIR code needs updating for BIO_n{read,write}{,0} removal. */
+/* #define USE_BIOPAIR */
+
 #define _BSD_SOURCE 1		/* Or gethostname won't be declared properly
 				   on Linux and GNU platforms. */
 #include <sys/types.h>
@@ -164,9 +167,6 @@
 #include <openssl/x509.h>
 #include <openssl/x509v3.h>
 #include <openssl/ssl.h>
-#ifndef OPENSSL_NO_ENGINE
-#include <openssl/engine.h>
-#endif
 #include <openssl/err.h>
 #include <openssl/rand.h>
 #include <openssl/rsa.h>
@@ -254,6 +254,7 @@ cb_server_alpn(SSL *s, const unsigned char **out, unsigned char *outlen,
 	 * Make a copy of the selected protocol which will be freed in
 	 * verify_alpn.
 	 */
+	free(alpn_selected);
 	if ((alpn_selected = malloc(*outlen)) == NULL) {
 		fprintf(stderr, "malloc failed\n");
 		abort();
@@ -277,8 +278,8 @@ verify_alpn(SSL *client, SSL *server)
 	free(alpn_selected);
 	alpn_selected = NULL;
 
-	if (client_proto_len != server_proto_len ||
-	    memcmp(client_proto, server_proto, client_proto_len) != 0) {
+	if (client_proto_len != server_proto_len || (client_proto_len > 0 &&
+	    memcmp(client_proto, server_proto, client_proto_len) != 0)) {
 		BIO_printf(bio_stdout, "ALPN selected protocols differ!\n");
 		goto err;
 	}
@@ -336,7 +337,7 @@ sv_usage(void)
 	fprintf(stderr, " -dhe1024dsa   - use 1024 bit key (with 160-bit subprime) for DHE\n");
 	fprintf(stderr, " -no_dhe       - disable DHE\n");
 	fprintf(stderr, " -no_ecdhe     - disable ECDHE\n");
-	fprintf(stderr, " -dtls1        - use DTLSv1\n");
+	fprintf(stderr, " -dtls1_2      - use DTLSv1.2\n");
 	fprintf(stderr, " -tls1         - use TLSv1\n");
 	fprintf(stderr, " -tls1_2       - use TLSv1.2\n");
 	fprintf(stderr, " -CApath arg   - PEM format directory of CA's\n");
@@ -409,7 +410,7 @@ main(int argc, char *argv[])
 	int badop = 0;
 	int bio_pair = 0;
 	int force = 0;
-	int tls1 = 0, tls1_2 = 0, dtls1 = 0, ret = 1;
+	int tls1 = 0, tls1_2 = 0, dtls1_2 = 0, ret = 1;
 	int client_auth = 0;
 	int server_auth = 0, i;
 	char *app_verify_arg = "Test Callback Argument";
@@ -464,8 +465,8 @@ main(int argc, char *argv[])
 			no_dhe = 1;
 		else if (strcmp(*argv, "-no_ecdhe") == 0)
 			no_ecdhe = 1;
-		else if (strcmp(*argv, "-dtls1") == 0)
-			dtls1 = 1;
+		else if (strcmp(*argv, "-dtls1_2") == 0)
+			dtls1_2 = 1;
 		else if (strcmp(*argv, "-tls1") == 0)
 			tls1 = 1;
 		else if (strcmp(*argv, "-tls1_2") == 0)
@@ -565,7 +566,7 @@ bad:
 		goto end;
 	}
 
-	if (!dtls1 && !tls1 && !tls1_2 && number > 1 && !reuse && !force) {
+	if (!dtls1_2 && !tls1 && !tls1_2 && number > 1 && !reuse && !force) {
 		fprintf(stderr,
 		    "This case cannot work.  Use -f to perform "
 		    "the test anyway (and\n-d to see what happens), "
@@ -588,8 +589,8 @@ bad:
 	SSL_library_init();
 	SSL_load_error_strings();
 
-	if (dtls1)
-		meth = DTLSv1_method();
+	if (dtls1_2)
+		meth = DTLSv1_2_method();
 	else if (tls1)
 		meth = TLSv1_method();
 	else if (tls1_2)
@@ -713,10 +714,12 @@ bad:
 	for (i = 0; i < number; i++) {
 		if (!reuse)
 			SSL_set_session(c_ssl, NULL);
+#ifdef USE_BIOPAIR
 		if (bio_pair)
 			ret = doit_biopair(s_ssl, c_ssl, bytes, &s_time,
 			    &c_time);
 		else
+#endif
 			ret = doit(s_ssl, c_ssl, bytes);
 	}
 
@@ -756,20 +759,17 @@ end:
 	SSL_CTX_free(c_ctx);
 	BIO_free(bio_stdout);
 
-#ifndef OPENSSL_NO_ENGINE
-	ENGINE_cleanup();
-#endif
 	CRYPTO_cleanup_all_ex_data();
 	ERR_free_strings();
 	ERR_remove_thread_state(NULL);
 	EVP_cleanup();
-	CRYPTO_mem_leaks(bio_err);
 	BIO_free(bio_err);
 
 	exit(ret);
 	return ret;
 }
 
+#if USE_BIOPAIR
 int
 doit_biopair(SSL *s_ssl, SSL *c_ssl, long count, clock_t *s_time,
     clock_t *c_time)
@@ -1113,6 +1113,7 @@ err:
 
 	return ret;
 }
+#endif
 
 
 #define W_READ	1
